#!/usr/bin/env python

# (c) Copyright [2020 - 2021] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (Micro Focus) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

import os
import json
import gettext
import fcntl
import commands
from collections import OrderedDict

#Write the health of DSfW service to /var/opt/novell/xad/dsfw.json
def writeJSON(dictionary):
	try:
		with open("/var/opt/novell/xad/dsfw.json", "w") as dsfwstatusfile:
			fcntl.flock(dsfwstatusfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
			json.dump(dictionary, dsfwstatusfile, indent=4, separators=(", ", ": "))
			fcntl.flock(dsfwstatusfile, fcntl.LOCK_UN)
	except IOError:
		raise Exception("Failed to open/write the Json file /var/opt/novell/xad/dsfw.json")

#To Get DSfW Health status data
def generateHealthStatus():

	jsondsfw_stream = OrderedDict(
		[
			('serviceName', 'Domain Services for Windows (DSfW)'),
			('status', ''),
			('children',
				[OrderedDict(
				[
					('serviceName', 'ndsd'),
					('status', ''),
					('runningSince' , ''),
					('AdditionalInfo', ''),
					('children', None)
				]),
				OrderedDict(
				[
					('serviceName', 'novell-named'),
					('status', ''),
					('runningSince' , ''),
					('AdditionalInfo', ''),
					('children', None)
				]),
				OrderedDict(
                                [
                                        ('serviceName', 'nscd'),
                                        ('status', ''),
					('runningSince' , ''),
					('AdditionalInfo', ''),
                                        ('children', None)
                                ]),
				OrderedDict(
                                [
                                        ('serviceName', 'rpcd'),
                                        ('status', ''),
					('runningSince' , ''),
					('AdditionalInfo', ''),
                                        ('children', None)
                                ]),
				OrderedDict(
                                [
                                        ('serviceName', 'Kerberos'),
                                        ('status', ''),
					('AdditionalInfo', ''),
                                        ('children', [
						OrderedDict(
                                                [
                                                        ('serviceName', 'xad-krb5kdc'),
                                                        ('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
                                                        ('children', None)
                                                ]),
                                                OrderedDict(
                                                [
                                                        ('serviceName', 'xad-kpasswdd'),
                                                        ('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
                                                        ('children', None)
                                                ])
					])
                                ]),
				OrderedDict(
                                [
                                        ('serviceName', 'xadsd'),
                                        ('status', ''),
					('runningSince' , ''),
					('AdditionalInfo', ''),
                                        ('children', None)
                                ]),
				OrderedDict(
                                [
					('serviceName', 'Samba'),
                                        ('status', ''),
					('AdditionalInfo', ''),
                                        ('children', [
						OrderedDict(
		                                [
							('serviceName', 'nmb'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
		                                ]),
						OrderedDict(
						[
							('serviceName', 'winbind'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						]),
						OrderedDict(
						[
							('serviceName', 'smb'),
							('status', ''),
							('runningSince' , ''),
							('AdditionalInfo', ''),
							('children', None)
						])
					])
                                ])
				]
			)
		])

	ndsdstatus = commands.getoutput("timeout 60s systemctl is-active ndsd")
	novell_namedstatus = commands.getoutput("timeout 60s systemctl is-active novell-named")
	nscdstatus = commands.getoutput("timeout 60s systemctl is-active nscd")
	rpcdstatus = commands.getoutput("timeout 60s systemctl is-active rpcd")
	xad_krb5kdcstatus = commands.getoutput("timeout 60s systemctl is-active xad-krb5kdc")
	xad_kpasswddstatus = commands.getoutput("timeout 60s systemctl is-active xad-kpasswdd")
	xadsdstatus = commands.getoutput("timeout 60s systemctl is-active xadsd")
	nmbstatus = commands.getoutput("timeout 60s systemctl is-active nmb")
	winbindstatus = commands.getoutput("timeout 60s systemctl is-active winbind")
	smbstatus = commands.getoutput("timeout 60s systemctl is-active smb")

	if(ndsdstatus == "active"):
		ndsdtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp ndsd")
		timestamp_output = ndsdtimestatus.strip().split("=")
		jsondsfw_stream["children"][0]["status"] = ("Healthy")
		jsondsfw_stream["children"][0]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][0]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][0]["runningSince"] = "0"
		jsondsfw_stream["children"][0]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/eDirectory/log/ndsd.log"]

	if(nscdstatus == "active"):
                nscdtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp nscd")
                timestamp_output = nscdtimestatus.strip().split("=")
                jsondsfw_stream["children"][2]["status"] = ("Healthy")
                jsondsfw_stream["children"][2]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][2]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][2]["runningSince"] = "0"
		jsondsfw_stream["children"][2]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages"]

	if(rpcdstatus == "active"):
		rpcdtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp rpcd")
		timestamp_output = rpcdtimestatus.strip().split("=")
		jsondsfw_stream["children"][3]["status"] = ("Healthy")
		jsondsfw_stream["children"][3]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][3]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][3]["runningSince"] = "0"
		jsondsfw_stream["children"][3]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages"]


	if(xad_krb5kdcstatus == "active"):
		xad_krb5kdctimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp rpcd")
		timestamp_output = xad_krb5kdctimestatus.strip().split("=")
		jsondsfw_stream["children"][4]["children"][0]["status"] = ("Healthy")
		jsondsfw_stream["children"][4]["children"][0]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][4]["children"][0]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][4]["children"][0]["runningSince"] = "0"
		jsondsfw_stream["children"][4]["children"][0]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/xad/log/kdc.log"]

	if(xad_kpasswddstatus == "active"):
		xad_kpasswddtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp xad-kpasswdd")
		timestamp_output = xad_kpasswddtimestatus.strip().split("=")
		jsondsfw_stream["children"][4]["children"][1]["status"] = ("Healthy")
		jsondsfw_stream["children"][4]["children"][1]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][4]["children"][1]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][4]["children"][1]["runningSince"] = "0"
		jsondsfw_stream["children"][4]["children"][1]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/opt/novell/xad/log/kpasswdd.log"]

	if(xadsdstatus == "active"):
		xadsdtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp xadsd")
		timestamp_output = xadsdtimestatus.strip().split("=")
		jsondsfw_stream["children"][5]["status"] = ("Healthy")
		jsondsfw_stream["children"][5]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][5]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][5]["runningSince"] = "0"
		jsondsfw_stream["children"][5]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages"]

	if(nmbstatus == "active"):
		nmbtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp nmb")
		timestamp_output = nmbtimestatus.strip().split("=")
		jsondsfw_stream["children"][6]["children"][0]["status"] = ("Healthy")
		jsondsfw_stream["children"][6]["children"][0]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][6]["children"][0]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][6]["children"][0]["runningSince"] = "0"
		jsondsfw_stream["children"][6]["children"][0]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.nmbd"]

	if(winbindstatus == "active"):
		winbindtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp winbind")
		timestamp_output = winbindtimestatus.strip().split("=")
		jsondsfw_stream["children"][6]["children"][1]["status"] = ("Healthy")
		jsondsfw_stream["children"][6]["children"][1]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][6]["children"][1]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][6]["children"][1]["runningSince"] = "0"
		jsondsfw_stream["children"][6]["children"][1]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.winbindd"]

	if(smbstatus == "active"):
		smbtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp smb")
		timestamp_output = smbtimestatus.strip().split("=")
		jsondsfw_stream["children"][6]["children"][2]["status"] = ("Healthy")
		jsondsfw_stream["children"][6]["children"][2]["runningSince"] = commands.getoutput("date --date=\""+timestamp_output[1]+"\" +\"%s\"")
	else:
		jsondsfw_stream["children"][6]["children"][2]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][6]["children"][2]["runningSince"] = "0"
		jsondsfw_stream["children"][6]["children"][2]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/messages", "/var/log/samba/log.smbd"]
	if( (jsondsfw_stream["children"][4]["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["children"][1]["status"] == ("Healthy")) ):
		jsondsfw_stream["children"][4]["status"] = ("Healthy")
	else:
		jsondsfw_stream["children"][4]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][4]["AdditionalInfo"] = ["Common_DaemonService1", "Common_DaemonService2"]

	if( (jsondsfw_stream["children"][6]["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["children"][1]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["children"][2]["status"] == ("Healthy")) ):
		jsondsfw_stream["children"][6]["status"] = ("Healthy")
	else:
		jsondsfw_stream["children"][6]["status"] = ("UnHealthy")
		jsondsfw_stream["children"][6]["AdditionalInfo"] = ["Common_DaemonService1", "Common_DaemonService2"]

	DNS_Master = commands.getoutput("/opt/novell/xad/share/dcinit/printConfigKey.pl \"DNS Master\"")
	if(DNS_Master == "TRUE"):
		if(novell_namedstatus == "active"):
			novell_namedtimestatus = commands.getoutput("systemctl show -p ExecMainStartTimestamp novell-named")
			timestamp_output = novell_namedtimestatus.strip().split("=")
			jsondsfw_stream["children"][1]["status"] = ("Healthy")
			jsondsfw_stream["children"][1]["runningSince"] = timestamp_output[1]
		else:
			jsondsfw_stream["children"][1]["status"] = ("UnHealthy")
			jsondsfw_stream["children"][1]["runningSince"] = "0"
			jsondsfw_stream["children"][1]["AdditionalInfo"] = ["Common_ReferLogMoreInfo", "/var/log/message", "/var/opt/novell/log/named/named.run"]

		if( (jsondsfw_stream["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][1]["status"] == ("Healthy")) and (jsondsfw_stream["children"][2]["status"] == ("Healthy")) and (jsondsfw_stream["children"][3]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["status"] == ("Healthy")) and (jsondsfw_stream["children"][5]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["status"] == ("Healthy")) ):
                        jsondsfw_stream["status"] = ("Healthy")
                else:
                        jsondsfw_stream["status"] = ("UnHealthy")
	else:
		if( (jsondsfw_stream["children"][0]["status"] == ("Healthy")) and (jsondsfw_stream["children"][2]["status"] == ("Healthy")) and (jsondsfw_stream["children"][3]["status"] == ("Healthy")) and (jsondsfw_stream["children"][4]["status"] == ("Healthy")) and (jsondsfw_stream["children"][5]["status"] == ("Healthy")) and (jsondsfw_stream["children"][6]["status"] == ("Healthy")) ):
			jsondsfw_stream["status"] = ("Healthy")
		else:
			jsondsfw_stream["status"] = ("UnHealthy")

		jsondsfw_stream["children"].remove(jsondsfw_stream["children"][1])

	return jsondsfw_stream

dsfwHealthData = generateHealthStatus()

try:
	writeJSON(dsfwHealthData)
except Exception:
        exit(1)
